!SFX_LIC Copyright 1994-2014 CNRS, Meteo-France and Universite Paul Sabatier
!SFX_LIC This is part of the SURFEX software governed by the CeCILL-C licence
!SFX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt  
!SFX_LIC for details. version 1.
!     #########
      SUBROUTINE DIAG_ISBA_INIT_n (CHI, DE, DEC, NDE, NDEC, DGO, D, DC, ND, NDC,  &
                                   OREAD_BUDGETC, NGB, GB, IO, NP, HSNOW_SCHEME, &
                                   KABC, HPROGRAM, KLU, KSW)
!     #####################
!
!!****  *DIAG_ISBA_INIT_n* - routine to initialize ISBA diagnostic variables
!!
!!    PURPOSE
!!    -------
!!
!!**  METHOD
!!    ------
!!
!!    EXTERNAL
!!    --------
!!
!!
!!    IMPLICIT ARGUMENTS
!!    ------------------
!!
!!    REFERENCE
!!    ---------
!!
!!
!!    AUTHOR
!!    ------
!!      V. Masson   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original    02/2003 
!!      modified    11/2003 by P. LeMoigne: surface cumulated energy budget
!!      modified    10/2004 by P. LeMoigne: surface miscellaneous fields
!!      B. Decharme    2008    New diag for water budget and allow to reset
!                               cumulatives variables at the beginning of a run
!!      B. Decharme 06/2009    add patch budget switch 
!!      B. Decharme 08/2009    add cummulative diag
!!      A.L. Gibelin 04/2009 : Add respiration diagnostics
!!      A.L. Gibelin 05/2009 : Add carbon spinup
!!      A.L. Gibelin 07/2009 : Suppress RDK and transform GPP as a diagnostic
!!      B. Decharme  05/12   : Carbon fluxes in diag_evap
!!      B. Decharme  10/12     Isba water budget diag
!!      B. Decharme    10/2012 : New diag for DIF:
!!                               F2 stress
!!                               Root zone swi, wg and wgi
!!                               swi, wg and wgi comparable to ISBA-FR-DG2 and DG3 layers
!!                               active layer thickness over permafrost
!!                               frozen layer thickness over non-permafrost
!!      B. Vincendon   02/2014 : condition added for RAD_BUDGET variables (needed for
!!                               restart mode)
!       B. Decharme    04/2013 : Add new diag for coupling
!                                Delete XTSRAD (because same than XTSRAD_NAT)
!!      P. Samuelsson  10/2014 : MEB
!!      P. Hagenmuller 09/2017 : Mepra
!!      A. Druel       02/2019 : Changes LARGIP flag to LIRRIGMODE flog for irrigation paramaters... (and delete XSEUIL)
!!      B. Decharme    02/2022 : Rewrite the routine and split misc diag for more simplicity
!!
!-------------------------------------------------------------------------------
!
!*       0.0    DECLARATIONS
!              -------------
!
USE MODE_DIAG
!
USE MODD_TYPE_DATE_SURF
!
USE MODD_CH_ISBA_n,        ONLY : CH_ISBA_t
USE MODD_DIAG_EVAP_ISBA_n, ONLY : DIAG_EVAP_ISBA_t, DIAG_EVAP_ISBA_NP_t
USE MODD_DIAG_n,           ONLY : DIAG_t, DIAG_NP_t, DIAG_OPTIONS_t
USE MODD_GR_BIOG_n,        ONLY : GR_BIOG_NP_t, GR_BIOG_t
USE MODD_ISBA_OPTIONS_n,   ONLY : ISBA_OPTIONS_t
USE MODD_ISBA_n,           ONLY : ISBA_NP_t, ISBA_P_t
!
USE MODD_SURF_PAR,         ONLY : XUNDEF, LEN_HREC
USE MODD_AGRI,             ONLY : LIRRIGMODE
!
#ifdef SFX_OL
USE MODN_IO_OFFLINE,       ONLY : LRESTART
#endif
!
USE MODI_PACK_SAME_RANK
USE MODI_MAKE_CHOICE_ARRAY
USE MODI_READ_SURF
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE
!
!*       0.01   Declarations of arguments
!              -------------------------
!
TYPE(CH_ISBA_t),           INTENT(INOUT) :: CHI
TYPE(DIAG_EVAP_ISBA_t),    INTENT(INOUT) :: DE
TYPE(DIAG_EVAP_ISBA_t),    INTENT(INOUT) :: DEC
TYPE(DIAG_EVAP_ISBA_NP_t), INTENT(INOUT) :: NDE
TYPE(DIAG_EVAP_ISBA_NP_t), INTENT(INOUT) :: NDEC
TYPE(DIAG_OPTIONS_t),      INTENT(IN   ) :: DGO
TYPE(DIAG_t),              INTENT(INOUT) :: D
TYPE(DIAG_t),              INTENT(INOUT) :: DC
TYPE(DIAG_NP_t),           INTENT(INOUT) :: ND
TYPE(DIAG_NP_t),           INTENT(INOUT) :: NDC
TYPE(GR_BIOG_NP_t),        INTENT(INOUT) :: NGB
TYPE(GR_BIOG_t),           INTENT(INOUT) :: GB
TYPE(ISBA_OPTIONS_t),      INTENT(INOUT) :: IO
TYPE(ISBA_NP_t),           INTENT(INOUT) :: NP
!
LOGICAL,          INTENT(IN) :: OREAD_BUDGETC
CHARACTER(LEN=*), INTENT(IN) :: HSNOW_SCHEME
INTEGER,          INTENT(IN) :: KABC
INTEGER,          INTENT(IN) :: KLU       ! size of arrays
INTEGER,          INTENT(IN) :: KSW       ! spectral bands
CHARACTER(LEN=6), INTENT(IN) :: HPROGRAM  ! program calling
!
!*       0.02   Declarations of local variables
!              -------------------------------
!
TYPE(DIAG_OPTIONS_t)   :: YDO
TYPE(DIAG_EVAP_ISBA_t) :: YDE
!
LOGICAL                 :: GCUMUL, GREAD_RESTART
INTEGER                 :: JP
INTEGER                 :: IVERSION, IBUG
INTEGER                 :: IRESP            ! IRESP  : return-code if a problem appears
INTEGER                 :: ISIZE_LMEB_PATCH ! Number of patches where multi-energy balance should be applied
CHARACTER(LEN=LEN_HREC) :: YREC             ! Name of the article to be read
REAL(KIND=JPRB)         :: ZHOOK_HANDLE
!
!-------------------------------------------------------------------------------
!
!/////////////////ATTRIBUTES INITIALISATIONS //////////////////////////
!
IF (LHOOK) CALL DR_HOOK('DIAG_ISBA_INIT_N',0,ZHOOK_HANDLE)
!
ISIZE_LMEB_PATCH=COUNT(IO%LMEB_PATCH(:))
!
YDO%LSURF_BUDGET = .TRUE.
YDO%N2M          = 1
YDO%LCOEF        = .TRUE.
YDO%LSURF_VARS   = .TRUE.
!
YDE%LSURF_EVAP_BUDGET = .TRUE.
!
GREAD_RESTART= .FALSE.
!
#ifdef SFX_OL
GCUMUL = (DGO%LSURF_BUDGETC .OR. (LRESTART .AND. .NOT.DGO%LRESET_BUDGETC))
#else
GCUMUL = (DGO%LSURF_BUDGETC .OR. .NOT.DGO%LRESET_BUDGETC)
#endif
!
IF(GCUMUL)THEN
  !
  CALL READ_SURF(HPROGRAM,'VERSION',IVERSION,IRESP)
  CALL READ_SURF(HPROGRAM,'BUG ',IBUG,IRESP)
  !
  GREAD_RESTART=(OREAD_BUDGETC .OR. (OREAD_BUDGETC.AND.(.NOT.DGO%LRESET_BUDGETC)))
  !
ENDIF
!
!-------------------------------------------------------------------------------
!
!////////////////NOT CUMULATED DIAGNOSTICS//////////////////////////////
!
!* surface energy budget
!
CALL ALLOC_BUD(YDO,D,KLU,KSW)
!
ALLOCATE(D%XEVAP (KLU))
ALLOCATE(D%XSUBL (KLU))
ALLOCATE(D%XSFCO2(KLU))
D%XEVAP  = XUNDEF
D%XSUBL  = XUNDEF
D%XSFCO2 = XUNDEF
!
DO JP = 1,IO%NPATCH
  !
  CALL ALLOC_BUD(YDO,ND%AL(JP),NP%AL(JP)%NSIZE_P,KSW)
  !
  ALLOCATE(ND%AL(JP)%XCDN  (NP%AL(JP)%NSIZE_P))
  ALLOCATE(ND%AL(JP)%XHUG  (NP%AL(JP)%NSIZE_P))
  ALLOCATE(ND%AL(JP)%XHU   (NP%AL(JP)%NSIZE_P))
  ALLOCATE(ND%AL(JP)%XEVAP (NP%AL(JP)%NSIZE_P))
  ALLOCATE(ND%AL(JP)%XSUBL (NP%AL(JP)%NSIZE_P))  
  ALLOCATE(ND%AL(JP)%XSFCO2(NP%AL(JP)%NSIZE_P))
  !
  ND%AL(JP)%XCDN   = XUNDEF
  ND%AL(JP)%XHUG   = XUNDEF
  ND%AL(JP)%XHU    = XUNDEF
  ND%AL(JP)%XEVAP  = XUNDEF
  ND%AL(JP)%XSUBL  = XUNDEF
  ND%AL(JP)%XSFCO2 = XUNDEF
  !
ENDDO
!
!* transfer coefficients
!
ALLOCATE(D%XZ0EFF(KLU))
D%XZ0EFF = XUNDEF  
!
DO JP=1,IO%NPATCH
  ALLOCATE(ND%AL(JP)%XZ0EFF(NP%AL(JP)%NSIZE_P))
  ND%AL(JP)%XZ0EFF = XUNDEF
ENDDO
!
!* surface temperature and parameters at 2m
!
ALLOCATE(D%XALBT(KLU))
D%XALBT = XUNDEF
!
DO JP=1,IO%NPATCH
  ALLOCATE(ND%AL(JP)%XTS    (NP%AL(JP)%NSIZE_P))
  ALLOCATE(ND%AL(JP)%XALBT  (NP%AL(JP)%NSIZE_P))
  ALLOCATE(ND%AL(JP)%XTSRAD (NP%AL(JP)%NSIZE_P))
  ND%AL(JP)%XTS    = XUNDEF
  ND%AL(JP)%XTSRAD = XUNDEF
  ND%AL(JP)%XALBT  = XUNDEF
ENDDO
!
!* detailed surface energy budget
!     
ALLOCATE(DE%XRAINFALL  (KLU))
ALLOCATE(DE%XSNOWFALL  (KLU))
DE%XRAINFALL   = XUNDEF
DE%XSNOWFALL   = XUNDEF
! 
CALL ALLOC_EVAP_DIAG  (DE,KLU,0)
CALL ALLOC_CC_DIAG    (DE,KLU)
CALL ALLOC_WATBUD_DIAG(DE,KLU)
CALL ALLOC_NRJBUD_DIAG(DE,KLU)
DO JP = 1,IO%NPATCH
  CALL ALLOC_EVAP_DIAG  (NDE%AL(JP),NP%AL(JP)%NSIZE_P,NP%AL(JP)%NSIZE_P)
  CALL ALLOC_CC_DIAG    (NDE%AL(JP),NP%AL(JP)%NSIZE_P)
  CALL ALLOC_WATBUD_DIAG(NDE%AL(JP),NP%AL(JP)%NSIZE_P)
  CALL ALLOC_NRJBUD_DIAG(NDE%AL(JP),NP%AL(JP)%NSIZE_P)
ENDDO
!
IF (ISIZE_LMEB_PATCH>0) THEN
   CALL ALLOC_MEB_DIAG(DE,KLU)
   DO JP = 1,IO%NPATCH
      CALL ALLOC_MEB_DIAG(NDE%AL(JP),NP%AL(JP)%NSIZE_P)
   ENDDO
ELSE
   CALL ALLOC_MEB_DIAG(DE,0)
   DO JP=1,IO%NPATCH
      CALL ALLOC_MEB_DIAG(NDE%AL(JP),0)
   ENDDO
ENDIF
!
!-------------------------------------------------------------------------------
!
!* Chemical fluxes
!
IF (CHI%SVI%NBEQ>0 .AND. CHI%LCH_BIO_FLUX) THEN
   !
   ALLOCATE(GB%XFISO(KLU))
   ALLOCATE(GB%XFMONO(KLU))
   !
   GB%XFISO  = XUNDEF
   GB%XFMONO = XUNDEF
ELSE
   ALLOCATE(GB%XFISO(0))
   ALLOCATE(GB%XFMONO(0))
ENDIF
!
IF (IO%CPHOTO/='NON') THEN
   !
   DO JP = 1,IO%NPATCH
      ALLOCATE(NGB%AL(JP)%XIACAN(NP%AL(JP)%NSIZE_P,KABC))
      NGB%AL(JP)%XIACAN = XUNDEF
   ENDDO
   !
ELSE
   !
   DO JP = 1,IO%NPATCH
      ALLOCATE(NGB%AL(JP)%XIACAN(0,0))
   ENDDO
   !
ENDIF
!
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!
!
!////////////////CUMULATED DIAGNOSTICS//////////////////////////////
!
!
IF(GCUMUL)THEN
  !
  !
  !///////////////////////ALLOCATIONS//////////////////////
  !
  !
  ! * Alloc global
  !
  ALLOCATE(DC%XEVAP(KLU))
  ALLOCATE(DC%XSUBL(KLU))
  !
  ALLOCATE(DEC%XRAINFALL(KLU))
  ALLOCATE(DEC%XSNOWFALL(KLU))
  !
  CALL ALLOC_SURF_BUD   (DC,0,KLU,0)
  CALL ALLOC_EVAP_DIAG  (DEC,KLU,0)
  CALL ALLOC_CC_DIAG    (DEC,KLU)
  CALL ALLOC_WATBUD_DIAG(DEC,KLU)
  CALL ALLOC_NRJBUD_DIAG(DEC,KLU)
  !
  IF (ISIZE_LMEB_PATCH>0) THEN
     CALL ALLOC_MEB_DIAG(DEC,KLU)
  ENDIF
  !
  ! * Alloc per patch
  !
  DO JP=1,IO%NPATCH
     !
     ALLOCATE(NDC%AL(JP)%XEVAP(NP%AL(JP)%NSIZE_P))
     ALLOCATE(NDC%AL(JP)%XSUBL(NP%AL(JP)%NSIZE_P))
     !
     CALL ALLOC_SURF_BUD   (NDC%AL(JP),0,NP%AL(JP)%NSIZE_P,0)  
     CALL ALLOC_EVAP_DIAG  (NDEC%AL(JP),NP%AL(JP)%NSIZE_P,0)
     CALL ALLOC_CC_DIAG    (NDEC%AL(JP),NP%AL(JP)%NSIZE_P)
     CALL ALLOC_WATBUD_DIAG(NDEC%AL(JP),NP%AL(JP)%NSIZE_P)
     CALL ALLOC_NRJBUD_DIAG(NDEC%AL(JP),NP%AL(JP)%NSIZE_P)
     !
  ENDDO
  !
  IF (ISIZE_LMEB_PATCH>0) THEN
     DO JP=1,IO%NPATCH
        CALL ALLOC_MEB_DIAG(NDEC%AL(JP),NP%AL(JP)%NSIZE_P)
     ENDDO
  ENDIF
  !
  !
  !/////////////////INITIALISATIONS//////////////////////////
  !
  !
  ! * Init global
  !
  DC%XEVAP = 0.
  DC%XSUBL = 0.
  !
  IF (GREAD_RESTART.AND.(IVERSION>7.OR.IVERSION==7.AND.IBUG>=3)) THEN 
     YREC='RAINFC_ISBA'
     CALL READ_SURF(HPROGRAM,YREC,DEC%XRAINFALL,IRESP)
     YREC='SNOWFC_ISBA'
     CALL READ_SURF(HPROGRAM,YREC,DEC%XSNOWFALL,IRESP)
  ELSE
     DEC%XRAINFALL = 0.0
     DEC%XSNOWFALL = 0.0
  ENDIF

  CALL INIT_SURF_BUD  (DC,0.)
  CALL INIT_EVAP_BUD  (DEC)
  CALL INIT_CC_BUD    (DEC,IO)
  CALL INIT_WATER_BUD (DEC)
  CALL INIT_ENERGY_BUD(DEC)
  !
  IF (ISIZE_LMEB_PATCH>0) THEN
     CALL INIT_MEB_BUD(DEC)
  ENDIF
  !
  ! * Init per patch
  !
  DO JP=1,IO%NPATCH
     !
     NDC%AL(JP)%XEVAP = 0.
     NDC%AL(JP)%XSUBL = 0.
     !
     CALL INIT_SURF_BUD(NDC%AL(JP),0.)
     !
     CALL INIT_EVAP_BUD  (NDEC%AL(JP))
     CALL INIT_CC_BUD    (NDEC%AL(JP),IO)
     CALL INIT_WATER_BUD (NDEC%AL(JP))
     CALL INIT_ENERGY_BUD(NDEC%AL(JP))
     !
  ENDDO
  !
  IF (ISIZE_LMEB_PATCH>0) THEN
     DO JP=1,IO%NPATCH
        CALL INIT_MEB_BUD(NDEC%AL(JP))
     ENDDO
  ENDIF
  !
  !
  ! * OPTIONAL Read restart file per path
  !
  !
  IF(GREAD_RESTART .AND. DGO%LPATCH_BUDGET .AND. IO%NPATCH>1)THEN
    !
    CALL READ_RESTART_PATCH(IO%NPATCH)
    !
  ENDIF
  !
  !
ELSE
  !
  !///////////////////////NO CUMUL//////////////////////
  !
  ! * NO GCUMUL global
  !
  ALLOCATE(DC%XEVAP(0))
  ALLOCATE(DC%XSUBL(0)) 
  !
  ALLOCATE(DEC%XRAINFALL(0))
  ALLOCATE(DEC%XSNOWFALL(0))
  !
  CALL ALLOC_SURF_BUD(DC,0,0,0)
  CALL ALLOC_EVAP_DIAG(DEC,0,0)
  CALL ALLOC_CC_DIAG    (DEC,0)
  CALL ALLOC_WATBUD_DIAG(DEC,0)
  CALL ALLOC_NRJBUD_DIAG(DEC,0)
  !
  IF (ISIZE_LMEB_PATCH>0) THEN
     CALL ALLOC_MEB_DIAG(DEC,0)  
  ENDIF
  !
  ! * NO GCUMUL per patch
  !
  DO JP=1,IO%NPATCH
    !
    ALLOCATE(NDC%AL(JP)%XEVAP(0))
    ALLOCATE(NDC%AL(JP)%XSUBL(0))
    !
    CALL ALLOC_SURF_BUD(NDC%AL(JP),0,0,0)     
    !
    CALL ALLOC_EVAP_DIAG(NDEC%AL(JP),0,0)
    CALL ALLOC_CC_DIAG    (NDEC%AL(JP),0)
    CALL ALLOC_WATBUD_DIAG(NDEC%AL(JP),0)
    CALL ALLOC_NRJBUD_DIAG(NDEC%AL(JP),0)
    !
  ENDDO
  !
  IF (ISIZE_LMEB_PATCH>0) THEN
     DO JP=1,IO%NPATCH
        CALL ALLOC_MEB_DIAG(NDEC%AL(JP),0)
     ENDDO
  ENDIF
  !
ENDIF
!
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!
IF (LHOOK) CALL DR_HOOK('DIAG_ISBA_INIT_N',1,ZHOOK_HANDLE)
!
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
CONTAINS
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!
SUBROUTINE ALLOC_EVAP_DIAG(DEA,KLUA,KLUAP)
!
TYPE(DIAG_EVAP_ISBA_t), INTENT(INOUT) :: DEA
INTEGER, INTENT(IN) :: KLUA
INTEGER, INTENT(IN) :: KLUAP
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
IF (LHOOK) CALL DR_HOOK('DIAG_ISBA_INIT_N:ALLOC_EVAP_DIAG',0,ZHOOK_HANDLE)
!
!////////////DIAG DEFINED BY PATCH AND AVERAGED//////////
!
ALLOCATE(DEA%XEPOT      (KLUA))
!
ALLOCATE(DEA%XLEG       (KLUA))
ALLOCATE(DEA%XLEGI      (KLUA))
ALLOCATE(DEA%XLEV       (KLUA))
ALLOCATE(DEA%XLES       (KLUA))
!
ALLOCATE(DEA%XLESL      (KLUA))
ALLOCATE(DEA%XSNDRIFT   (KLUA))
ALLOCATE(DEA%XSNFREE_SWU(KLUA))
!
ALLOCATE(DEA%XLER       (KLUA))
ALLOCATE(DEA%XLETR      (KLUA))
!
ALLOCATE(DEA%XDRAIN     (KLUA))
ALLOCATE(DEA%XRUNOFF    (KLUA))
ALLOCATE(DEA%XDRIP      (KLUA))
ALLOCATE(DEA%XRRVEG     (KLUA))
ALLOCATE(DEA%XMELT      (KLUA))
ALLOCATE(DEA%XMELTSTOT  (KLUA))
ALLOCATE(DEA%XSNREFREEZ (KLUA))
ALLOCATE(DEA%XICEFLUX   (KLUA))
!
ALLOCATE(DEA%XIRRIG_FLUX(KLUA))
!
ALLOCATE(DEA%XQSB       (KLUA))
ALLOCATE(DEA%XHORT      (KLUA)) 
!
ALLOCATE(DEA%XIFLOOD   (KLUA))
ALLOCATE(DEA%XPFLOOD   (KLUA))
ALLOCATE(DEA%XLE_FLOOD (KLUA))
ALLOCATE(DEA%XLEI_FLOOD(KLUA))
!
ALLOCATE(DEA%XSWNET_N  (KLUA))
ALLOCATE(DEA%XSWNET_NS (KLUA))
ALLOCATE(DEA%XLWNET_N  (KLUA))
!
IF (KLUA>0) THEN
  !
  DEA%XEPOT       = XUNDEF
  !
  DEA%XLEG        = XUNDEF
  DEA%XLEGI       = XUNDEF
  DEA%XLEV        = XUNDEF
  DEA%XLES        = XUNDEF
  !
  DEA%XLESL       = XUNDEF
  DEA%XSNDRIFT    = XUNDEF
  DEA%XSNFREE_SWU = XUNDEF
  !  
  DEA%XLER        = XUNDEF
  DEA%XLETR       = XUNDEF
  !
  DEA%XDRAIN      = XUNDEF
  DEA%XRUNOFF     = XUNDEF
  DEA%XDRIP       = XUNDEF
  DEA%XRRVEG      = XUNDEF
  DEA%XMELT       = XUNDEF
  DEA%XMELTSTOT   = XUNDEF
  DEA%XSNREFREEZ  = XUNDEF
  DEA%XICEFLUX    = XUNDEF
  !  
  DEA%XIRRIG_FLUX = XUNDEF
  !
  DEA%XQSB        = XUNDEF
  DEA%XHORT       = XUNDEF
  !  
  DEA%XIFLOOD     = XUNDEF
  DEA%XPFLOOD     = XUNDEF
  DEA%XLE_FLOOD   = XUNDEF
  DEA%XLEI_FLOOD  = XUNDEF
  !
  DEA%XSWNET_N    = XUNDEF
  DEA%XSWNET_NS   = XUNDEF
  DEA%XLWNET_N    = XUNDEF
  !
ENDIF
!
!//////////////DIAG DEFINED ONLY BY PATCH//////////////
!
ALLOCATE(DEA%XRN_SN_FR   (KLUAP))
ALLOCATE(DEA%XH_SN_FR    (KLUAP))
ALLOCATE(DEA%XLEI_SN_FR  (KLUAP))
ALLOCATE(DEA%XLE_SN_FR   (KLUAP))
ALLOCATE(DEA%XGFLUX_SN_FR(KLUAP))
ALLOCATE(DEA%XLEG_SN_FR  (KLUAP))
ALLOCATE(DEA%XLEGI_SN_FR (KLUAP))
ALLOCATE(DEA%XLEV_SN_FR  (KLUAP))
ALLOCATE(DEA%XLETR_SN_FR (KLUAP))
ALLOCATE(DEA%XUSTAR_SN_FR(KLUAP))
ALLOCATE(DEA%XLER_SN_FR  (KLUAP))
!
ALLOCATE(DEA%XMELTADV    (KLUAP))
!
IF (KLUAP>0) THEN
  DEA%XRN_SN_FR    = XUNDEF
  DEA%XH_SN_FR     = XUNDEF
  DEA%XLEI_SN_FR   = XUNDEF
  DEA%XLE_SN_FR    = XUNDEF
  DEA%XGFLUX_SN_FR = XUNDEF
  DEA%XLEG_SN_FR   = XUNDEF
  DEA%XLEGI_SN_FR  = XUNDEF
  DEA%XLEV_SN_FR   = XUNDEF
  DEA%XLETR_SN_FR  = XUNDEF
  DEA%XUSTAR_SN_FR = XUNDEF
  DEA%XLER_SN_FR   = XUNDEF
  !
  DEA%XMELTADV     = XUNDEF  
  !
ENDIF
!
IF (LHOOK) CALL DR_HOOK('DIAG_ISBA_INIT_N:ALLOC_EVAP_DIAG',1,ZHOOK_HANDLE)
!
END SUBROUTINE ALLOC_EVAP_DIAG
!
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!
SUBROUTINE ALLOC_CC_DIAG(DEA,KLUA)
!
TYPE(DIAG_EVAP_ISBA_t), INTENT(INOUT) :: DEA
INTEGER, INTENT(IN) :: KLUA
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
IF (LHOOK) CALL DR_HOOK('DIAG_ISBA_INIT_N:ALLOC_CC_DIAG',0,ZHOOK_HANDLE)
!
! * General diag
!
ALLOCATE(DEA%XGPP      (KLUA))
ALLOCATE(DEA%XRESP_AUTO(KLUA))
ALLOCATE(DEA%XRESP_ECO (KLUA))
ALLOCATE(DEA%XTURNVTOT (KLUA))
ALLOCATE(DEA%XFLTOSCARB(KLUA))
ALLOCATE(DEA%XRESPSCARB(KLUA))
ALLOCATE(DEA%XRESPLIT  (KLUA))
!
IF (KLUA>0) THEN
  DEA%XGPP        = XUNDEF
  DEA%XRESP_AUTO  = XUNDEF
  DEA%XRESP_ECO   = XUNDEF 
  DEA%XTURNVTOT   = XUNDEF 
  DEA%XFLTOSCARB  = XUNDEF
  DEA%XRESPSCARB  = XUNDEF
  DEA%XRESPLIT    = XUNDEF
ENDIF
!
! * Carbon leaching diag
!
IF(IO%LCLEACH)THEN
  !
  ALLOCATE(DEA%XFDOC   (KLUA))
  ALLOCATE(DEA%XFDOCLIT(KLUA))
  !
  IF (KLUA>0) THEN
    DEA%XFDOC    = XUNDEF
    DEA%XFDOCLIT = XUNDEF
  ENDIF
  !
ELSE
  !
  ALLOCATE(DEA%XFDOC   (0))
  ALLOCATE(DEA%XFDOCLIT(0))
  !
ENDIF
!
! * Biomass fire diag
!
IF(IO%LFIRE)THEN
  !
  ALLOCATE(DEA%XFIRETURNOVER(KLUA))
  ALLOCATE(DEA%XFIRECO2     (KLUA))
  ALLOCATE(DEA%XFIREBCS     (KLUA))
  !
  IF (KLUA>0) THEN
    DEA%XFIRETURNOVER = XUNDEF
    DEA%XFIRECO2      = XUNDEF
    DEA%XFIREBCS      = XUNDEF
  ENDIF
  !
ELSE
  !
  ALLOCATE(DEA%XFIRETURNOVER(0))
  ALLOCATE(DEA%XFIRECO2     (0))
  ALLOCATE(DEA%XFIREBCS     (0))
  !
ENDIF
!
! * Land use land cover diag
!
IF(IO%LLULCC)THEN
  ALLOCATE(DEA%XFHARVEST(KLUA))
  IF (KLUA>0) THEN
    DEA%XFHARVEST = XUNDEF
  ENDIF
ELSE
  ALLOCATE(DEA%XFHARVEST(0))
ENDIF
!
! * Soil gas scheme diag
!
IF(IO%LSOILGAS)THEN
  !
  ALLOCATE(DEA%XO2FLUX  (KLUA))
  ALLOCATE(DEA%XCH4FLUX (KLUA))
  ALLOCATE(DEA%XSURF_O2 (KLUA))
  ALLOCATE(DEA%XSURF_CO2(KLUA))
  ALLOCATE(DEA%XSURF_CH4(KLUA))
  ALLOCATE(DEA%XEVAP_O2 (KLUA))
  ALLOCATE(DEA%XEVAP_CO2(KLUA))
  ALLOCATE(DEA%XEVAP_CH4(KLUA))
  ALLOCATE(DEA%XPMT_O2  (KLUA))
  ALLOCATE(DEA%XPMT_CO2 (KLUA))
  ALLOCATE(DEA%XPMT_CH4 (KLUA))
  ALLOCATE(DEA%XEBU_CH4 (KLUA))
  !
  ALLOCATE(DEA%XFCONS_O2 (KLUA))
  ALLOCATE(DEA%XFPROD_CO2(KLUA))
  ALLOCATE(DEA%XFMT_CH4  (KLUA))
  ALLOCATE(DEA%XFMG_CH4  (KLUA))
  !
  IF (KLUA>0) THEN
    !
    DEA%XO2FLUX   = XUNDEF
    DEA%XCH4FLUX  = XUNDEF
    DEA%XSURF_O2  = XUNDEF
    DEA%XSURF_CO2 = XUNDEF
    DEA%XSURF_CH4 = XUNDEF
    DEA%XEVAP_O2  = XUNDEF
    DEA%XEVAP_CO2 = XUNDEF
    DEA%XEVAP_CH4 = XUNDEF
    DEA%XPMT_O2   = XUNDEF
    DEA%XPMT_CO2  = XUNDEF
    DEA%XPMT_CH4  = XUNDEF
    DEA%XEBU_CH4  = XUNDEF
    !
    DEA%XFCONS_O2  = XUNDEF
    DEA%XFPROD_CO2 = XUNDEF
    DEA%XFMT_CH4   = XUNDEF
    DEA%XFMG_CH4   = XUNDEF
    !
  ENDIF
  !
ELSE
  !
  ALLOCATE(DEA%XO2FLUX  (0))
  ALLOCATE(DEA%XCH4FLUX (0))
  ALLOCATE(DEA%XSURF_O2 (0))
  ALLOCATE(DEA%XSURF_CO2(0))
  ALLOCATE(DEA%XSURF_CH4(0))
  ALLOCATE(DEA%XEVAP_O2 (0))
  ALLOCATE(DEA%XEVAP_CO2(0))
  ALLOCATE(DEA%XEVAP_CH4(0))
  ALLOCATE(DEA%XPMT_O2  (0))
  ALLOCATE(DEA%XPMT_CO2 (0))
  ALLOCATE(DEA%XPMT_CH4 (0))
  ALLOCATE(DEA%XEBU_CH4 (0))
  !
  ALLOCATE(DEA%XFCONS_O2 (0))
  ALLOCATE(DEA%XFPROD_CO2(0))
  ALLOCATE(DEA%XFMT_CH4  (0))
  ALLOCATE(DEA%XFMG_CH4  (0))
  !
ENDIF
!
IF (LHOOK) CALL DR_HOOK('DIAG_ISBA_INIT_N:ALLOC_CC_DIAG',1,ZHOOK_HANDLE)
!
END SUBROUTINE ALLOC_CC_DIAG
!
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!
SUBROUTINE ALLOC_MEB_DIAG(DEA,KLUA)
!
TYPE(DIAG_EVAP_ISBA_t), INTENT(INOUT) :: DEA
INTEGER, INTENT(IN) :: KLUA
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
IF (LHOOK) CALL DR_HOOK('DIAG_ISBA_INIT_N:ALLOC_MEB_DIAG',0,ZHOOK_HANDLE)
!
ALLOCATE(DEA%XLELITTER (KLUA))
ALLOCATE(DEA%XLELITTERI(KLUA))
ALLOCATE(DEA%XDRIPLIT  (KLUA))
ALLOCATE(DEA%XRRLIT  (KLUA))
!
ALLOCATE(DEA%XLEV_CV (KLUA))
ALLOCATE(DEA%XLES_CV (KLUA))
ALLOCATE(DEA%XLETR_CV(KLUA))
ALLOCATE(DEA%XLER_CV (KLUA))
ALLOCATE(DEA%XLE_CV  (KLUA))
ALLOCATE(DEA%XH_CV   (KLUA))
ALLOCATE(DEA%XMELT_CV(KLUA))
ALLOCATE(DEA%XFRZ_CV (KLUA))

ALLOCATE(DEA%XLETR_GV(KLUA))
ALLOCATE(DEA%XLER_GV (KLUA))
ALLOCATE(DEA%XLE_GV  (KLUA))
ALLOCATE(DEA%XH_GV   (KLUA))

ALLOCATE(DEA%XLE_GN  (KLUA))
ALLOCATE(DEA%XEVAP_GN(KLUA))
ALLOCATE(DEA%XH_GN   (KLUA))
ALLOCATE(DEA%XSR_GN  (KLUA))
ALLOCATE(DEA%XSWDOWN_GN(KLUA))
ALLOCATE(DEA%XLWDOWN_GN(KLUA))

ALLOCATE(DEA%XEVAP_G (KLUA))
ALLOCATE(DEA%XLE_CA  (KLUA))
ALLOCATE(DEA%XH_CA   (KLUA))
!
ALLOCATE(DEA%XSWUP(KLUA))
ALLOCATE(DEA%XLWUP(KLUA))
!
ALLOCATE(DEA%XSWNET_V    (KLUA))
ALLOCATE(DEA%XSWNET_G    (KLUA))
ALLOCATE(DEA%XLWNET_V    (KLUA))
ALLOCATE(DEA%XLWNET_G    (KLUA))
!
IF (KLUA>0) THEN
  !
  DEA%XLELITTER      = XUNDEF
  DEA%XLELITTERI     = XUNDEF
  DEA%XDRIPLIT       = XUNDEF
  DEA%XRRLIT         = XUNDEF
  !        
  DEA%XLEV_CV      = XUNDEF
  DEA%XLES_CV      = XUNDEF
  DEA%XLETR_CV     = XUNDEF
  DEA%XLER_CV      = XUNDEF
  DEA%XLE_CV       = XUNDEF  
  DEA%XH_CV        = XUNDEF  
  DEA%XMELT_CV     = XUNDEF
  DEA%XFRZ_CV      = XUNDEF  
  !
  DEA%XLETR_GV     = XUNDEF
  DEA%XLER_GV      = XUNDEF
  DEA%XLE_GV       = XUNDEF
  DEA%XH_GV        = XUNDEF
  !
  DEA%XLE_GN       = XUNDEF
  DEA%XEVAP_GN     = XUNDEF
  DEA%XH_GN        = XUNDEF  
  DEA%XSR_GN       = XUNDEF
  DEA%XSWDOWN_GN   = XUNDEF
  DEA%XLWDOWN_GN   = XUNDEF
  !
  DEA%XEVAP_G      = XUNDEF
  DEA%XLE_CA       = XUNDEF
  DEA%XH_CA        = XUNDEF
  !
  DEA%XSWUP = XUNDEF
  DEA%XLWUP = XUNDEF
  !
  DEA%XSWNET_V       = XUNDEF
  DEA%XSWNET_G       = XUNDEF
  DEA%XLWNET_V       = XUNDEF
  DEA%XLWNET_G       = XUNDEF
  DEA%XLWNET_N       = XUNDEF
  !
ENDIF
!
IF (LHOOK) CALL DR_HOOK('DIAG_ISBA_INIT_N:ALLOC_MEB_DIAG',1,ZHOOK_HANDLE)
!
END SUBROUTINE ALLOC_MEB_DIAG
!
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!
SUBROUTINE ALLOC_WATBUD_DIAG(DEA,KLUA)
!
TYPE(DIAG_EVAP_ISBA_t), INTENT(INOUT) :: DEA
INTEGER, INTENT(IN) :: KLUA
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
IF (LHOOK) CALL DR_HOOK('DIAG_ISBA_INIT_N:ALLOC_WATBUD_DIAG',0,ZHOOK_HANDLE)
!
ALLOCATE(DEA%XDWG    (KLUA))
ALLOCATE(DEA%XDWGI   (KLUA))
ALLOCATE(DEA%XDWR    (KLUA))
ALLOCATE(DEA%XDSWE   (KLUA))
ALLOCATE(DEA%XDSWFREE(KLUA))
ALLOCATE(DEA%XWATBUD (KLUA))
!
IF (KLUA>0) THEN
  DEA%XDWG    = XUNDEF
  DEA%XDWGI   = XUNDEF
  DEA%XDWR    = XUNDEF
  DEA%XDSWE   = XUNDEF
  DEA%XDSWFREE= XUNDEF
  DEA%XWATBUD = XUNDEF
ENDIF
!
IF (LHOOK) CALL DR_HOOK('DIAG_ISBA_INIT_N:ALLOC_WATBUD_DIAG',1,ZHOOK_HANDLE)
!
END SUBROUTINE ALLOC_WATBUD_DIAG
!
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!
SUBROUTINE ALLOC_NRJBUD_DIAG(DEA,KLUA)
!
TYPE(DIAG_EVAP_ISBA_t), INTENT(INOUT) :: DEA
INTEGER, INTENT(IN) :: KLUA
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
IF (LHOOK) CALL DR_HOOK('DIAG_ISBA_INIT_N:ALLOC_NRJBUD_DIAG',0,ZHOOK_HANDLE)
!
ALLOCATE(DEA%XNRJBUD       (KLUA))
ALLOCATE(DEA%XNRJBUD_SFC   (KLUA))
ALLOCATE(DEA%XGRNDFLUX     (KLUA))
ALLOCATE(DEA%XRESTORE      (KLUA))
ALLOCATE(DEA%XRESTOREN     (KLUA))
ALLOCATE(DEA%XDELHEATG     (KLUA))
ALLOCATE(DEA%XDELHEATN     (KLUA))
ALLOCATE(DEA%XDELPHASEG    (KLUA))
ALLOCATE(DEA%XDELPHASEN    (KLUA))
ALLOCATE(DEA%XDELHEATG_SFC (KLUA))
ALLOCATE(DEA%XDELHEATN_SFC (KLUA))
ALLOCATE(DEA%XDELPHASEG_SFC(KLUA))
ALLOCATE(DEA%XDELPHASEN_SFC(KLUA))
!
IF (KLUA>0) THEN
  DEA%XNRJBUD        = XUNDEF
  DEA%XNRJBUD_SFC    = XUNDEF
  DEA%XGRNDFLUX      = XUNDEF
  DEA%XRESTORE       = XUNDEF
  DEA%XRESTOREN      = XUNDEF
  DEA%XDELHEATG      = XUNDEF
  DEA%XDELHEATN      = XUNDEF
  DEA%XDELPHASEG     = XUNDEF
  DEA%XDELPHASEN     = XUNDEF
  DEA%XDELHEATG_SFC  = XUNDEF
  DEA%XDELHEATN_SFC  = XUNDEF
  DEA%XDELPHASEG_SFC = XUNDEF
  DEA%XDELPHASEN_SFC = XUNDEF
ENDIF
!
IF (LHOOK) CALL DR_HOOK('DIAG_ISBA_INIT_N:ALLOC_NRJBUD_DIAG',1,ZHOOK_HANDLE)
!
END SUBROUTINE ALLOC_NRJBUD_DIAG
!
!-------------------------------------------------------------------------------
!-------------------------------------------------------------------------------
!
SUBROUTINE READ_RESTART_PATCH(KNP)
!
!* arguments
!
INTEGER, INTENT(IN)      :: KNP
!
!* local variables
!
REAL, DIMENSION(KLU,KNP) :: ZWORK
!
LOGICAL          :: GDIM
CHARACTER(LEN=6) :: YREC2
!
REAL(KIND=JPRB)  :: ZHOOK_HANDLE
!
!* main code
!
IF (LHOOK) CALL DR_HOOK('DIAG_ISBA_INIT_N:READ_RESTART_PATCH',0,ZHOOK_HANDLE)
!
ZWORK(:,:) = XUNDEF
!
GDIM = (IVERSION>8 .OR. IVERSION==8 .AND. IBUG>0)
!
IF (GDIM) CALL READ_SURF(HPROGRAM,'SPLIT_PATCH',GDIM,IRESP) 
!
IF (GDIM) THEN
   YREC2=''
ELSEIF (IVERSION<7 .OR. IVERSION==7 .AND. IBUG<3) THEN
   YREC2='PATCH'
ELSE
   YREC2='P'
ENDIF
!
YREC='LEGC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLEG(:))
ENDDO
!
YREC='LEGIC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLEGI(:))
ENDDO
!
YREC='LEVC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLEV(:))
ENDDO
!
YREC='LESC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLES(:))
ENDDO
!
IF (HSNOW_SCHEME=='3-L' .OR. HSNOW_SCHEME=='CRO') THEN
   !
   IF(IVERSION>7 .OR. IVERSION==7 .AND. IBUG>=3)THEN
     !
     YREC='LESLC_'//YREC2
     CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
     DO JP = 1,KNP
        CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLESL(:))
     ENDDO
     !
   ENDIF
   !
   IF(IVERSION>=8)THEN
     !
     YREC='SNDRIFC_'//YREC2  
     CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
     DO JP = 1,KNP
        CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XSNDRIFT(:))
     ENDDO
     !
   ENDIF  
   !
ENDIF   
!
YREC='LERC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLER(:))
ENDDO
!
YREC='LETRC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLETR(:))
ENDDO 
!
YREC='EVAPC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDC%AL(JP)%XEVAP(:))
ENDDO
!
IF (IVERSION>=8)THEN
   YREC='SUBLC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDC%AL(JP)%XSUBL(:))
   ENDDO  
ENDIF   
! 
YREC='DRAINC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XDRAIN(:))
ENDDO   
!  
YREC='RUNOFFC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XRUNOFF(:))
ENDDO   
!  
YREC='DRIVEGC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XDRIP(:))
ENDDO
!  
YREC='RRVEGC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XRRVEG(:))
ENDDO  
!  
YREC='SNOMLTC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XMELT(:))
ENDDO  
!
IF (LIRRIGMODE) THEN
   YREC='IRRIGC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XIRRIG_FLUX(:))
   ENDDO  
ENDIF
!
IF (IO%CPHOTO/='NON' .AND. (IVERSION>7 .OR. IVERSION==7 .AND. IBUG>=3)) THEN
   !
   YREC='GPPC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XGPP(:))
   ENDDO 
   !
   YREC='RC_AUTO_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XRESP_AUTO(:))
   ENDDO
   !
   YREC='RC_ECO_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XRESP_ECO(:))
   ENDDO 
   !
ENDIF
!
IF ((IO%CRUNOFF=='SGH'.AND.IO%CISBA=='DIF').AND.IVERSION>=8) THEN
   YREC='QSBC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XQSB(:))
   ENDDO
ENDIF
!
IF (IO%CHORT=='SGH'.OR.IO%CISBA=='DIF')THEN
   YREC='HORTONC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XHORT(:))
   ENDDO  
ENDIF
!
IF (IO%LFLOOD) THEN
   !
   YREC='IFLOODC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XIFLOOD(:))
   ENDDO 
   !
   YREC='PFLOODC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XPFLOOD(:))
   ENDDO  
   !
   YREC='LEFC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLE_FLOOD(:))
   ENDDO  
   !
   YREC='LEIFC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLEI_FLOOD(:))
   ENDDO
   !
ENDIF  
!   
IF (ISIZE_LMEB_PATCH>0) THEN
   !
   YREC='LEV_CVC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLEV_CV(:))
   ENDDO
   !
   YREC='LES_CVC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLES_CV(:))
   ENDDO   
   !   
   YREC='LETR_CVC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLETR_CV(:))
   ENDDO   
   !   
   YREC='LER_CVC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLER_CV(:))
   ENDDO
   !   
   YREC='LE_CVC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLE_CV(:))
   ENDDO
   !   
   YREC='H_CVC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XH_CV(:))
   ENDDO  
   !   
   YREC='MELT_CVC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XMELT_CV(:))
   ENDDO
   !
   YREC='FRZ_CVC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XFRZ_CV(:))
   ENDDO
   ! 
   YREC='LE_GVC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLE_GV(:))
   ENDDO 
   !
   YREC='H_GVC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XH_GV(:))
   ENDDO  
   !  
   YREC='LE_GNC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLE_GN(:))
   ENDDO  
   !
   YREC='H_GNC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XH_GN(:))
   ENDDO  
   !   
   YREC='SR_GNC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XSR_GN(:))
   ENDDO  
   !
   YREC='SWDN_GNC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XSWDOWN_GN(:))
   ENDDO  
   !
   YREC='LWDN_GNC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLWDOWN_GN(:))
   ENDDO  
   !
   YREC='LE_CAC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLE_CA(:))
   ENDDO   
   !   
   YREC='H_CAC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XH_CA(:))
   ENDDO
   !  
   YREC='SWNT_VC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XSWNET_V(:))
   ENDDO  
   !
   YREC='SWNT_GC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XSWNET_G(:))
   ENDDO
   !
   YREC='SWNT_NC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XSWNET_N(:))
   ENDDO
   !
   YREC='SWNT_NSC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XSWNET_NS(:))
   ENDDO 
   !
   YREC='LWNT_VC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLWNET_V(:))
   ENDDO 
   !
   YREC='LWNT_GC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLWNET_G(:))
   ENDDO
   !
   YREC='LWNT_NC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XLWNET_N(:))
   ENDDO 
   ! 
ENDIF
!
YREC='RNC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDC%AL(JP)%XRN(:))
ENDDO
!
YREC='HC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDC%AL(JP)%XH(:))
ENDDO  
!  
YREC='LEC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDC%AL(JP)%XLE(:))
ENDDO   
!   
YREC='LEIC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDC%AL(JP)%XLEI(:))
ENDDO 
!   
YREC='GFLUXC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDC%AL(JP)%XGFLUX(:))
ENDDO 
!  
IF (DGO%LRAD_BUDGET) THEN
   !
   YREC='SWDC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDC%AL(JP)%XSWD(:))
   ENDDO
   !
   YREC='SWUC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDC%AL(JP)%XSWU(:))
   ENDDO 
   !
   YREC='LWDC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDC%AL(JP)%XLWD(:))
   ENDDO 
   !
   YREC='LWUC_'//YREC2
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDC%AL(JP)%XLWU(:))
   ENDDO
   !
ENDIF
!
YREC='FMUC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDC%AL(JP)%XFMU(:))
ENDDO
!  
YREC='FMVC_'//YREC2
CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
DO JP = 1,KNP
   CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDC%AL(JP)%XFMV(:))
ENDDO   
!
IF (IO%LGLACIER) THEN
   YREC='ICE_FC_'//YREC2 
   CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
   DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XICEFLUX(:))
   ENDDO   
ENDIF
!
#ifdef SFX_OL
IF (DE%LWATER_BUDGET .OR. (LRESTART .AND. .NOT.DGO%LRESET_BUDGETC)) THEN
#else
IF (DE%LWATER_BUDGET .OR. .NOT.DGO%LRESET_BUDGETC) THEN
#endif
   ! 
   IF (IVERSION>7 .OR. IVERSION==7 .AND. IBUG>=3) THEN 
      !
      YREC='DWGC_'//YREC2
      CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
      DO JP = 1,KNP
      CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XDWG(:))
      ENDDO
      !    
      YREC='DWGIC_'//YREC2
      CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
      DO JP = 1,KNP
          CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XDWGI(:))
      ENDDO
      !
      YREC='DWRC_'//YREC2
      CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
      DO JP = 1,KNP
         CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XDWR(:))
      ENDDO
      !        
      YREC='DSWEC_'//YREC2
      CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
      DO JP = 1,KNP
         CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XDSWE(:))
      ENDDO  
      !
      YREC='WATBUDC_'//YREC2
      CALL MAKE_CHOICE_ARRAY(HPROGRAM, KNP, GDIM, YREC, ZWORK)
      DO JP = 1,KNP
         CALL PACK_SAME_RANK(NP%AL(JP)%NR_P,ZWORK(:,JP),NDEC%AL(JP)%XWATBUD(:))
      ENDDO
      !       
   ENDIF
   !
ENDIF
!
IF (LHOOK) CALL DR_HOOK('DIAG_ISBA_INIT_N:READ_RESTART_PATCH',1,ZHOOK_HANDLE)
!
END SUBROUTINE READ_RESTART_PATCH
!
!-------------------------------------------------------------------------------
!
END SUBROUTINE DIAG_ISBA_INIT_n
