!SFX_LIC Copyright 1994-2014 CNRS, Meteo-France and Universite Paul Sabatier
!SFX_LIC This is part of the SURFEX software governed by the CeCILL-C licence
!SFX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt  
!SFX_LIC for details. version 1.
!     #########
      SUBROUTINE READ_PGD_TEB_n (DTCO, U, UG, GCP, TOP, SPAOP, TG, &
                                 BOP, BDD, DTB, DTT, HPROGRAM)
!     #########################################
!
!!****  *READ_PGD_TEB_n* - reads TEB physiographic fields
!!                       
!!
!!    PURPOSE
!!    -------
!!
!!**  METHOD
!!    ------
!!
!!    EXTERNAL
!!    --------
!!
!!
!!    IMPLICIT ARGUMENTS
!!    ------------------
!!
!!    REFERENCE
!!    ---------
!!
!!
!!    AUTHOR
!!    ------
!!      V. Masson   *Meteo France*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original    01/2003 
!!      M. Goret     02/2017 :add CO2 conversion factors
!!      M. Goret     04/2017 :add NTIME_CHANGE
!!      M. Goret     05/2017 :add traffic cycle
!-------------------------------------------------------------------------------
!
!*       0.    DECLARATIONS
!              ------------
!
USE MODD_DATA_COVER_n, ONLY : DATA_COVER_t
USE MODD_SURF_ATM_n, ONLY : SURF_ATM_t
USE MODD_SURF_ATM_GRID_n, ONLY : SURF_ATM_GRID_t
USE MODD_GRID_CONF_PROJ_n, ONLY : GRID_CONF_PROJ_t
USE MODD_SURF_PAR, ONLY : LEN_HREC
!
USE MODD_TEB_OPTION_n, ONLY : TEB_OPTIONS_t
USE MODD_SPARTACUS_OPTION_n, ONLY : SPARTACUS_OPTIONS_t
USE MODD_SFX_GRID_n, ONLY : GRID_t
USE MODD_BEM_OPTION_n, ONLY : BEM_OPTIONS_t
USE MODD_BLD_DESCRIPTION_n, ONLY : BLD_DESC_t
USE MODD_DATA_BEM_n, ONLY : DATA_BEM_t
USE MODD_DATA_TEB_n, ONLY : DATA_TEB_t
!
USE MODD_TYPE_DATE_SURF
!
USE MODD_DATA_COVER_PAR, ONLY : JPCOVER
!
!
USE MODE_READ_SURF_COV, ONLY : READ_SURF_COV
!
USE MODI_READ_SURF
USE MODI_PACK_INIT
USE MODI_READ_PGD_TEB_PAR_n
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
USE MODI_GET_TYPE_DIM_n
!
USE MODI_READ_LECOCLIMAP
USE MODI_ABOR1_SFX
!
IMPLICIT NONE
!
!*       0.1   Declarations of arguments
!              -------------------------
!
!
TYPE(DATA_COVER_t), INTENT(INOUT) :: DTCO
TYPE(SURF_ATM_t), INTENT(INOUT) :: U
TYPE(SURF_ATM_GRID_t), INTENT(INOUT) :: UG
TYPE(GRID_CONF_PROJ_t),INTENT(INOUT) :: GCP
!
TYPE(TEB_OPTIONS_t), INTENT(INOUT) :: TOP
TYPE(SPARTACUS_OPTIONS_t), INTENT(INOUT) :: SPAOP
TYPE(GRID_t), INTENT(INOUT) :: TG
TYPE(BEM_OPTIONS_t), INTENT(INOUT) :: BOP
TYPE(BLD_DESC_t), INTENT(INOUT) :: BDD
TYPE(DATA_BEM_t), INTENT(INOUT) :: DTB
TYPE(DATA_TEB_t), INTENT(INOUT) :: DTT
!
 CHARACTER(LEN=6),  INTENT(IN)  :: HPROGRAM ! calling program
!
!*       0.2   Declarations of local variables
!              -------------------------------
!
 CHARACTER(LEN=LEN_HREC) :: YRECFM         ! Name of the article to be read
INTEGER           :: IVERSION
INTEGER           :: IBUGFIX
INTEGER           :: IRESP          ! Error code after redding
INTEGER           :: JLAYER
CHARACTER(LEN=4 ) :: YLVL
!
LOGICAL           :: GECOSG
LOGICAL           :: GNEW_NAME  !Flag to version v9 for TEB version manage
!
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!-------------------------------------------------------------------------------
!
!* 1D physical dimension
!
IF (LHOOK) CALL DR_HOOK('READ_PGD_TEB_N',0,ZHOOK_HANDLE)
YRECFM='SIZE_TOWN'
 CALL GET_TYPE_DIM_n(DTCO, U, 'TOWN  ',TG%NDIM)
!
!*       2.     Other dimension initializations:
!               --------------------------------
!
 CALL READ_SURF(HPROGRAM,'VERSION',IVERSION,IRESP)
 CALL READ_SURF(HPROGRAM,'BUG',IBUGFIX,IRESP)
!
GNEW_NAME=(IVERSION>=9)
!
!* number of TEB patches
!
IF (IVERSION<7 .OR. IVERSION==7 .AND. IBUGFIX<=2) THEN
  TOP%NTEB_PATCH=1
ELSE
  YRECFM='TEB_PATCH'
  CALL READ_SURF(HPROGRAM,YRECFM,TOP%NTEB_PATCH,IRESP)
END IF
!
!
! New option of soil vertical discretization
!
!IF ((IVERSION==8 .AND. IBUGFIX>=2) .OR. IVERSION>8) THEN
IF (GNEW_NAME) THEN
  YRECFM='ROAD_GRID'
  CALL READ_SURF(HPROGRAM,YRECFM,TOP%CROAD_GRID,IRESP)
ELSE
  TOP%CROAD_GRID='LOW5  ' 
ENDIF
!
!*  soil grid (below road, buildings, and garden, the latter if DIF option is used for ISBA)
!
!IF ((IVERSION==8 .AND. IBUGFIX>=2) .OR. IVERSION>8) THEN
IF (GNEW_NAME) THEN
  CALL READ_SURF(HPROGRAM,'TEB_SOIL_LAY',TOP%NTEB_SOIL,IRESP)
  ALLOCATE(TOP%XTEB_SOILGRID(TOP%NTEB_SOIL))
  CALL READ_SURF(HPROGRAM,'ROAD_LAYER',TOP%NTEB_ROAD,IRESP)  
  DO JLAYER=1,TOP%NTEB_SOIL
    WRITE(YLVL,'(I2)') JLAYER
    YRECFM='TEB_SGRID'//ADJUSTL(YLVL(:LEN_TRIM(YLVL)))
    CALL READ_SURF(HPROGRAM,YRECFM,TOP%XTEB_SOILGRID(JLAYER),IRESP)
  END DO
ELSE
  TOP%NTEB_ROAD         = 4
  TOP%NTEB_SOIL         = 5
  ALLOCATE(TOP%XTEB_SOILGRID(TOP%NTEB_SOIL))
  TOP%XTEB_SOILGRID(1) = 0.01
  TOP%XTEB_SOILGRID(2) = 0.1
  TOP%XTEB_SOILGRID(3) = 0.5
  TOP%XTEB_SOILGRID(4) = 1.0
  TOP%XTEB_SOILGRID(5) = 3.0
  !
ENDIF
!
!* number of roof layers
!
YRECFM='ROOF_LAYER'
 CALL READ_SURF(HPROGRAM,YRECFM,TOP%NROOF_LAYER,IRESP)

YRECFM='WALL_LAYER'
 CALL READ_SURF(HPROGRAM,YRECFM,TOP%NWALL_LAYER,IRESP)
!
!* number of change of the legal time
!
!IF ((IVERSION==8 .AND. IBUGFIX>=2) .OR. IVERSION>8) THEN
IF (GNEW_NAME) THEN
   !
   YRECFM='NTIME_CHANGE'
   CALL READ_SURF(HPROGRAM,YRECFM,TOP%NTIME_CHANGE,IRESP)
   !
   YRECFM='TRAF_MONTHLY'
   CALL READ_SURF(HPROGRAM,YRECFM,TOP%XTRAF_MONTHLY,IRESP,YRECFM,'-')
   !
   YRECFM='TRAF_DAILY'
   CALL READ_SURF(HPROGRAM,YRECFM,TOP%XTRAF_DAILY,IRESP,YRECFM,'-')
   !
   YRECFM='TRAF_HOURLY'
   CALL READ_SURF(HPROGRAM,YRECFM,TOP%XTRAF_HOURLY,IRESP,YRECFM,'-')
   !
   YRECFM='POP_MONTHLY'
   CALL READ_SURF(HPROGRAM,YRECFM,TOP%XPOP_MONTHLY,IRESP,YRECFM,'-')
   !
   YRECFM='POP_DAILY'
   CALL READ_SURF(HPROGRAM,YRECFM,TOP%XPOP_DAILY,IRESP,YRECFM,'-')
   !
   YRECFM='POP_HOURLY'
   CALL READ_SURF(HPROGRAM,YRECFM,TOP%XPOP_HOURLY,IRESP,YRECFM,'-')
   !
ELSE
   !
   TOP%NTIME_CHANGE = 0.
   !
   TOP%XTRAF_MONTHLY = (/1,1,1,1,1,1,1,1,1,1,1,1/)
   TOP%XTRAF_DAILY = (/1.05,1.07,1.08,1.09,1.15,0.86,0.7/)
   TOP%XTRAF_HOURLY = (/(12./33),(12./33),(12./33),(12./33),(12./33),(12./33), &  
                        (12./33),(48./33),(48./33),(48./33),(48./33),(48./33), &
                        (48./33),(48./33),(48./33),(48./33),(48./33),(48./33), &
                        (48./33),(48./33),(48./33),(12./33),(12./33),(12./33)/)
   !
   TOP%XPOP_MONTHLY = (/1,1,1,1,1,1,1,1,1,1,1,1/)
   TOP%XPOP_DAILY   = (/1,1,1,1,1,1,1/)
   TOP%XPOP_HOURLY  = (/1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1/)
   !
ENDIF
!
!
!* type of averaging for Buildings (to allow ascendant compatibility)
!* type of Building Energy Model
!
IF (IVERSION<7 .OR.( IVERSION==7 .AND. IBUGFIX<=2)) THEN
  TOP%CBLD_ATYPE='ARI'
  TOP%CBEM = 'DEF'
ELSE
  YRECFM='BLD_ATYPE'
  CALL READ_SURF(HPROGRAM,YRECFM,TOP%CBLD_ATYPE,IRESP)
  YRECFM='BEM'
  CALL READ_SURF(HPROGRAM,YRECFM,TOP%CBEM,IRESP)
END IF
!
IF (TOP%CBEM=="BEM") THEN
  YRECFM='FLOOR_LAYER'
  CALL READ_SURF(HPROGRAM,YRECFM,BOP%NFLOOR_LAYER,IRESP)
  !IF ((IVERSION==8 .AND. IBUGFIX>=2) .OR. IVERSION>8) THEN
  IF (GNEW_NAME) THEN
    YRECFM='MASS_LAYER'
    CALL READ_SURF(HPROGRAM,YRECFM,BOP%NMASS_LAYER,IRESP)
    YRECFM='BEMCOMP'
    CALL READ_SURF(HPROGRAM,YRECFM,BOP%NBEMCOMP,IRESP)
  ELSE
      BOP%NBEMCOMP=1
      BOP%NMASS_LAYER=BOP%NFLOOR_LAYER
  ENDIF
  YRECFM='COOL_COIL'
  CALL READ_SURF(HPROGRAM,YRECFM,BOP%CCOOL_COIL,IRESP)
  YRECFM='HEAT_COIL'
  CALL READ_SURF(HPROGRAM,YRECFM,BOP%CHEAT_COIL,IRESP)
  YRECFM='AUTOSIZE'
  CALL READ_SURF(HPROGRAM,YRECFM,BOP%LAUTOSIZE,IRESP)
  !
  ! CO2 conversion factors
  !
  !IF ((IVERSION==8 .AND. IBUGFIX>=2) .OR. IVERSION>8) THEN
  IF (GNEW_NAME) THEN
    YRECFM='CF_ELEC'
    CALL READ_SURF(HPROGRAM,YRECFM,BOP%XCF_CO2_ELEC,IRESP)
    YRECFM='CF_GAS'
    CALL READ_SURF(HPROGRAM,YRECFM,BOP%XCF_CO2_GAS,IRESP)
    YRECFM='CF_FUEL'
    CALL READ_SURF(HPROGRAM,YRECFM,BOP%XCF_CO2_FUEL,IRESP)
    YRECFM='CF_OTHER'
    CALL READ_SURF(HPROGRAM,YRECFM,BOP%XCF_CO2_OTHER,IRESP)
  ELSE
    BOP%XCF_CO2_ELEC = 0
    BOP%XCF_CO2_GAS = 57E-9
    BOP%XCF_CO2_FUEL = 75E-9
    BOP%XCF_CO2_OTHER = 92E-9
  ENDIF
ELSE
  BOP%NBEMCOMP=1
ENDIF
!
!* Case of high vegetation
!
IF (.NOT. GNEW_NAME) THEN
  TOP%CURBTREE = 'NONE'
ELSE
  YRECFM='URBTREE'
  CALL READ_SURF(HPROGRAM,YRECFM,TOP%CURBTREE,IRESP)
END IF
!
!* Case of urban green roofs
!
IF (TOP%LGARDEN) THEN
  IF (IVERSION<7 .OR.( IVERSION==7 .AND. IBUGFIX<=2)) THEN
    TOP%LGREENROOF = .FALSE.
  ELSE
    YRECFM='LGREENROOF'
    CALL READ_SURF(HPROGRAM,YRECFM,TOP%LGREENROOF,IRESP)
  END IF
!
!* Case of urban hydrology
!
  !IF (IVERSION<8 .OR.( IVERSION==8 .AND. IBUGFIX<=1)) THEN
  IF (.NOT. GNEW_NAME) THEN
    TOP%LURBHYDRO = .FALSE.
  ELSE
    YRECFM='LURBHYDRO'
    CALL READ_SURF(HPROGRAM,YRECFM,TOP%LURBHYDRO,IRESP)
  END IF
!
ENDIF
!
!* Solar panels
!
IF (IVERSION<7 .OR.( IVERSION==7 .AND. IBUGFIX<=3)) THEN
  TOP%LSOLAR_PANEL = .FALSE.
ELSE
  YRECFM='SOLAR_PANEL'
  CALL READ_SURF(HPROGRAM,YRECFM,TOP%LSOLAR_PANEL,IRESP)
END IF
!
!* SPARTACUS-Surface radiation scheme
!
!IF ((IVERSION==8 .AND. IBUGFIX<2) .OR. IVERSION<8) THEN
IF (.NOT. GNEW_NAME) THEN
   !
   TOP%LSPARTACUS = .FALSE.
   !
   SPAOP%LDO_SW = .FALSE.
   SPAOP%LDO_LW = .FALSE.
   SPAOP%LUSE_SW_DIRECT_ALBEDO = .FALSE.
   SPAOP%LDO_VEGETATION = .FALSE.
   SPAOP%LDO_URBAN = .FALSE.
   SPAOP%N_VEGETATION_REGION_URBAN = 0
   SPAOP%N_VEGETATION_REGION_FOREST = 0
   SPAOP%NSW = 0
   SPAOP%NLW = 0
   SPAOP%N_STREAM_SW_URBAN = 0
   SPAOP%N_STREAM_LW_URBAN = 0
   SPAOP%N_STREAM_SW_FOREST = 0
   SPAOP%N_STREAM_LW_FOREST = 0
   SPAOP%LUSE_SYMMETRIC_VEGETATION_SCALE_URBAN  = .FALSE.
   SPAOP%LUSE_SYMMETRIC_VEGETATION_SCALE_FOREST = .FALSE.
   SPAOP%XVEGETATION_ISOLATION_FACTOR_URBAN  = 0.0
   SPAOP%XVEGETATION_ISOLATION_FACTOR_FOREST = 0.0
   SPAOP%XMIN_VEGETATION_FRACTION = 0.0
   SPAOP%XMIN_BUILDING_FRACTION   = 0.0
   !
ELSE
   !
   YRECFM="SPARTACUS"
   CALL READ_SURF(HPROGRAM,YRECFM,TOP%LSPARTACUS,IRESP)
   !
   YRECFM="DO_SW"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%LDO_SW,IRESP)
   !
   YRECFM="DO_LW"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%LDO_LW,IRESP)
   !
   YRECFM="SW_DIR_ALB"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%LUSE_SW_DIRECT_ALBEDO,IRESP)
   ! 
   YRECFM="DO_VEG"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%LDO_VEGETATION,IRESP)
   !
   YRECFM="DO_URBAN"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%LDO_URBAN,IRESP)
   !
   YRECFM="VEGREGURB"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%N_VEGETATION_REGION_URBAN,IRESP)
   !
   YRECFM="VEGREGFOR"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%N_VEGETATION_REGION_FOREST,IRESP)
   !
   YRECFM="NSW"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%NSW,IRESP)
   !
   YRECFM="NLW"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%NLW,IRESP)
   !
   YRECFM="STREAMSWURB"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%N_STREAM_SW_URBAN,IRESP)
   !
   YRECFM="STREAMLWURB"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%N_STREAM_LW_URBAN,IRESP)
   !
   YRECFM="STREAMSWFOR"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%N_STREAM_SW_FOREST,IRESP)
   !
   YRECFM="STREAMLWFOR"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%N_STREAM_LW_FOREST,IRESP)
   !
   YRECFM="SYMVEGURB"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%LUSE_SYMMETRIC_VEGETATION_SCALE_URBAN,IRESP)
   !
   YRECFM="SYMVEGFOR"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%LUSE_SYMMETRIC_VEGETATION_SCALE_FOREST,IRESP)
   !
   YRECFM="VEGISOURB"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%XVEGETATION_ISOLATION_FACTOR_URBAN,IRESP)
   !
   YRECFM="VEGISOFOR"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%XVEGETATION_ISOLATION_FACTOR_FOREST,IRESP)
   !
   YRECFM="MIN_VEGFR"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%XMIN_VEGETATION_FRACTION,IRESP)
   !
   YRECFM="MIN_BLDFR"
   CALL READ_SURF(HPROGRAM,YRECFM,SPAOP%XMIN_BUILDING_FRACTION,IRESP)
   !
END IF
!
!* Explicit calculation of longwave radiation
!
!IF ((IVERSION==8 .AND. IBUGFIX<2) .OR. IVERSION<8) THEN
IF (.NOT. GNEW_NAME) THEN
   TOP%LEXPLW = .FALSE.
ELSE
   YRECFM="EXPLW"
   CALL READ_SURF(HPROGRAM,YRECFM,TOP%LEXPLW,IRESP)
END IF
!
!* Energy budget closure check for TEB
!
!IF ((IVERSION==8 .AND. IBUGFIX<2) .OR. IVERSION<8) THEN
IF (.NOT. GNEW_NAME) THEN
   TOP%LCHECK_TEB = .FALSE.
   TOP%XEPS_BDGT_GLOB = 1.E-3
   TOP%XEPS_BDGT_FAC = 1.E-6
ELSE
   YRECFM="CHECK_TEB"
   CALL READ_SURF(HPROGRAM,YRECFM,TOP%LCHECK_TEB,IRESP)
   YRECFM="EPS_BUD_GLOB"
   CALL READ_SURF(HPROGRAM,YRECFM,TOP%XEPS_BDGT_GLOB,IRESP)
   YRECFM="EPS_BUD_FAC"
   CALL READ_SURF(HPROGRAM,YRECFM,TOP%XEPS_BDGT_FAC,IRESP)
END IF
!
!*       3.     Physiographic data fields:
!               -------------------------
!
!* cover classes
!
ALLOCATE(TOP%LCOVER(JPCOVER))
ALLOCATE(TOP%XZS(TG%NDIM))
ALLOCATE(TG%XLAT       (TG%NDIM))
ALLOCATE(TG%XLON       (TG%NDIM))
ALLOCATE(TG%XMESH_SIZE (TG%NDIM))
CALL PACK_INIT(DTCO,U,UG,HPROGRAM,'TOWN  ',TG, TOP%LCOVER,TOP%XCOVER,TOP%XZS )
!
!-------------------------------------------------------------------------------
!
!*       4.     Physiographic data fields not to be computed by ecoclimap
!               ---------------------------------------------------------
!
 CALL READ_LECOCLIMAP(HPROGRAM,TOP%LECOCLIMAP,GECOSG)
!
 CALL READ_PGD_TEB_PAR_n(DTCO, U, GCP, BDD, DTB, DTT, TG%NDIM, TOP, BOP, &
                         HPROGRAM,'-')
!
IF (LHOOK) CALL DR_HOOK('READ_PGD_TEB_N',1,ZHOOK_HANDLE)
!
!------------------------------------------------------------------------------
!
END SUBROUTINE READ_PGD_TEB_n
